﻿/*
This file is responsibile to keep functions that are used throughout the UI
common to different pages
*/
var UISettings = {
    MsgTimeToLiveMs: 15000,
    MaxCharTopMsg: 190,
    GetTopeMenuEverySeconds: 60
};

var angularSEP = {
    AppName: "security-platform-ngapp",
    Scope: {},
    AppInstance: null,
    InitialiseController: function (controller) {
        try {
            var scopeAssigned = angular.element($("[ng-controller='" + controller + "']")).scope();
            if (scopeAssigned == null) {
                this.AppInstance.controller(controller, function ($scope, $http) {
                    angularSEP.Scope[controller] = $scope;
                });
                angular.bootstrap($("[ng-controller='" + controller + "']"), [this.AppName]);

                return true;
            }
            else {
                angularSEP.Scope[controller] = scopeAssigned;
                return false;
            }
        }
        catch (err) {
            console.log("Initialise Angular Controller failed with " + err);
            return false;
        }
    },
    ControllerTemporaryAccessCard: "temporaryAccessCardCtrl",
    ControllerDevicesInPtcMng: "devicesInPtcMgmtCtrl",
    ControllerPatchCardDevs: "pcDevicesCtrl",
    ControllerDevicesinGrp: "devicesInGrpCtrl",
    ControllerThreatsInAV: "threatsInAVCtrl",
    ControllerThreatCard: "threatCardCtrl",
    ControllerDashboardPatches: "patchesCtrl",
    ControllerDashboardAV: "avDashboardCtrl",
    ControllerDashboardEPS: "epsDashboardCtrl",
    ControllerDevicePatches: "devicePatchesCtrl",
    ControllerGrpPatchPolicyVendors: "PatchVendorsCtrl",
    ControllerGrpPermissionsTab: "permissionsCtrl",
    ControllerSettingsAdmins: "adminsCtrl",
    ControllerRolesPermissions: "rolesCtrl",
    ControllerNetworkControl: "networkCtrl",
    ControllerActivityMonitor: "activityMonCtrl",
    ControllerNotifications: "notificationsCtrl",
    ControllerDeviceNotifications: "deviceNotificationsCtrl",
    ControllerBubbleNotifications: "notificationsBubbleCtrl",
    ControllerLicensingAVGroupsTags: "licAVGroupsCtrl",
    ControllerLicensingPMGroupsTags: "licPMGroupsCtrl",
    ControllerLicensingECGroupsTags: "licECGroupsCtrl",
    ControllerGrpAV: "GroupAVCtrl", // Used to hold data only
    ControllerGrpAVExclusionLists: "ExclusionListAVCtrl",
    ControllerDeviceAVExclusionList: "DeviceExclusionListAVCtrl",
    ControllerGrpAVServiceSettingsExclusionLists: "ExclusionListAVServiceSettingsCtrl",
    ControllerEndPointControlServiceSettings: "EndPointControlServiceSettingsController",
    ControllerECGroupPolicyRule: "ECGroupPolicyRuleController",
    ControllerECGroupProtectionPolicy: "ECGroupProtectionPolicyController",
    ControllerEndPointControlDeviceTab: "EndPointControlDeviceTabController",
    ControllerGrpNotifications: "GroupNotificationsCtrl",
    ControllerPMService: "ServicePMCtrl",
    ControllerTemplateReports: "TemplateReportsCtrl",
    ControllerDeviceAlertCard: "alertCardCtrl",
    ControllerNagwareMsgs: "NagwareMsgsCtrl",
    ControllerDeviceHardware: "deviceHardwareCtrl",
    ControllerDeviceSoftware: "deviceSoftwareCtrl",
    ControllerMisc: "miscCtrl",
    ControllerPlatformUpdates: "updatesCtrl",
    ControllerATService: "ServiceATCtrl",
    ControllerGettingStarted: "GettingStartedCtrl",
}

var cookieKeys = {
    NagwareReminderDate: 'NagwareReminderDate'
}


angularSEP.AppInstance = angular.module(angularSEP.AppName, []);

angularSEP.AppInstance.directive("compileHtml", function ($parse, $sce, $compile) {
    return {
        restrict: 'A',
        link: function (scope, element, attributes) {
            $(element).replaceWith(attributes.htmltocompile);
        }
    }
});

angularSEP.AppInstance.directive('icheckCheckbox', function () {
    return {
        // Restrict it to be an attribute in this case
        restrict: 'A',
        // responsible for registering DOM listeners as well as updating the DOM
        link: function ($scope, element, $attrs) {
            var $element = $(element);
            $element.icheck('destroy');
            $element.icheck($scope.$eval($attrs.icheckCheckbox));

            $scope.$watch($attrs.ngDisabled, function (newValue) {
                if (newValue) {
                    $element.icheck('disabled');
                } else {
                    $element.icheck('enabled');
                }
            });
        }
    };
});

// This is the directive for daterange input. It will show the date range control which allows user to select a
// start date and an end date with some handy links on the left like Today, Yesterday, Last 7 Days and so on.
// to use it just write: <input type="daterange" ng-model="someModel" ranges="ranges"> where the 'ranges' object provides the
// shortcuts on the left column like Today, Yesterday, Last 7 Days and so on.
// It will put the selected start and end dates in someModel AS A STRING in this format: "{'start':'YYYY-MM-DD','end':'YYYY-MM-DD'}"
angularSEP.AppInstance.directive('input', ['$compile', '$parse', '$filter', function ($compile, $parse, $filter) {
    return {
        restrict: 'E',
        require: '?ngModel',
        link: function ($scope, $element, $attributes, ngModel) {

            if ($attributes.type !== 'daterange' || ngModel === null) {
                return;
            }

            var options = {};
            options.format = $attributes.format || 'MM/DD/YYYY';
            options.separator = $attributes.separator || ' - ';
            options.minDate = $attributes.minDate && moment($attributes.minDate);
            options.maxDate = $attributes.maxDate && moment($attributes.maxDate);
            options.dateLimit = $attributes.limit && moment.duration.apply(this, $attributes.limit.split(' ').map(function (elem, index) {
                return index === 0 && parseInt(elem, 10) || elem;
            }));
            options.ranges = $attributes.ranges && $parse($attributes.ranges)($scope);
            options.locale = $attributes.locale && $parse($attributes.locale)($scope);
            options.opens = $attributes.opens || $parse($attributes.opens)($scope);

            if ($attributes.enabletimepicker) {
                options.timePicker = true;
                angular.extend(options, $parse($attributes.enabletimepicker)($scope));
            }

            function datify(date) {
                return moment.isMoment(date) ? date.toDate() : date;
            }

            function momentify(date) {
                return (!moment.isMoment(date)) ? moment(date) : date;
            }

            function format(date) {
                return $filter('date')(datify(date), options.format.replace(/Y/g, 'y').replace(/D/g, 'd')); //date.format(options.format);
            }

            function formatted(dates) {
                return [format(dates.startDate), format(dates.endDate)].join(options.separator);
            }

            ngModel.$render = function () {
                if (!ngModel.$viewValue || !ngModel.$viewValue.startDate) {
                    return;
                }
                $element.val(formatted(ngModel.$viewValue));
            };

            $scope.$watch(function () {
                return $attributes.ngModel;
            }, function (modelValue, oldModelValue) {

                if (!ngModel.$modelValue || (!ngModel.$modelValue.startDate)) {
                    ngModel.$setViewValue({
                        startDate: moment().startOf('day'),
                        endDate: moment().startOf('day')
                    });
                    return;
                }

                if (oldModelValue !== modelValue) {
                    return;
                }

                $element.data('daterangepicker').startDate = momentify(ngModel.$modelValue.startDate);
                $element.data('daterangepicker').endDate = momentify(ngModel.$modelValue.endDate);
                $element.data('daterangepicker').updateView();
                $element.data('daterangepicker').updateCalendars();
                $element.data('daterangepicker').updateInputText();

            });

            $element.daterangepicker(options, function (start, end, label) {

                var modelValue = ngModel.$viewValue;

                if (angular.equals(start, modelValue.startDate) && angular.equals(end, modelValue.endDate)) {
                    return;
                }

                $scope.$apply(function () {
                    ngModel.$setViewValue({
                        startDate: (moment.isMoment(modelValue.startDate)) ? start : start.toDate(),
                        endDate: (moment.isMoment(modelValue.endDate)) ? end : end.toDate()
                    });
                    ngModel.$render();
                });

            });

        }

    };
}]);

function RegisterChangePageCleanupEvent() {
    $(".nav li a, .nav-dropdown li a, .mobile-side-menu-link").off("click");
    $(".nav li a, .nav-dropdown li a, .mobile-side-menu-link").on("click", function (e) {

        //Only the ones having AJAX calls
        if ($(this).attr("data-ajax-complete") != null) {
            hideUILoader();
            showPageContentLoader(LoadingPage);
            cardsLoaded = [];
            RevertThemeIfNotSaved();
            clearInterval(activityProgress);

            $("#btnSelectGroup").unbind();
            $("#updateContent").unbind();
        }

        if ($(this).hasClass("mobile-side-menu-link"))
            hideSideNavCustom();
    });
}

function ValidEmailAddress(emailAddress) {
    var regex = /^([a-zA-Z0-9_.+-])+\@(([a-zA-Z0-9-])+\.)+([a-zA-Z0-9]{2,4})+$/;
    return regex.test(emailAddress);
}

function ValidPort(port) {
    if ($.isNumeric(port)) {
        port = parseInt(port);
        var maxPort = parseInt('65535');
        if (port > maxPort || port == 0)
            return false;
        else
            return true;
    }
    else
        return false;
}

function clamp(msg, maxsize) {
    if (msg) {
        if (msg.length > maxsize)
            return msg.substring(0, maxsize - 1) + "...";
    }

    return msg;
}

function stopProbagation(event) {
    if (event.stopPropagation) { // W3C/addEventListener()
        event.stopPropagation();
    } else { // Older IE.
        event.cancelBubble = true;
    }
}

var cardTypeEnum = { patchCard: "patch", deviceCard: "device", notificationCard: "notification" }

function AddCardToHistory(cardType, cardVal) {
    if ((cardsLoaded.length > 0) && (cardsLoaded[cardsLoaded.length - 1].type == cardType) && (cardsLoaded[cardsLoaded.length - 1].value == cardVal))
        console.log("Skip adding same card to history");
    else
        cardsLoaded.push({ type: cardType, value: cardVal, title: $(".card-overlay.current").find(".card-header-title").text() })
}

function LoadPreviousCard() {
    if (cardsLoaded.length > 1) {
        cardsLoaded.splice(cardsLoaded.length - 1, 1);

        if (cardsLoaded[cardsLoaded.length - 1].type == cardTypeEnum.patchCard) {
            if (!$("#patch-card").is(":visible")) {
                openCard(selCards.patchCard)
                GetPatchCardData(cardsLoaded[cardsLoaded.length - 1].value);
            }
        }
        else if (cardsLoaded[cardsLoaded.length - 1].type == cardTypeEnum.deviceCard) {
            if (!$("#device-card").is(":visible")) {
                openCard(selCards.deviceCard);
                LoadDeviceInfo(cardsLoaded[cardsLoaded.length - 1].value);
            }
        }
        else if (cardsLoaded[cardsLoaded.length - 1].type == cardTypeEnum.notificationCard) {
            if (!$("#notification-card").is(":visible")) {
                openCard(selCards.notificationCard);
                GetNotification(cardsLoaded[cardsLoaded.length - 1].value);
            }
        }

        $(".card-overlay.current").find(".card-header-title").text(cardsLoaded[cardsLoaded.length - 1].title);
        $(selCards.overlay).addClass("current");

        if (cardsLoaded.length == 1)
            $(selCards.btnBackCard).hide();
        else
            $(selCards.btnBackCard).show();
    }

}

function ResetCardsHistory() {
    if (typeof GroupGuidsUpdated === 'undefined')
        InitialiseTables('#updateContent', { fnCategoriesUpdated: CategoriesUpdated });
    else
        InitialiseTables('#updateContent', {
            fnCategoriesUpdated: CategoriesUpdated,
            //fnGrpGuidsUpdated: GroupGuidsUpdated
        });

    cardsLoaded = [];
}

function SetTreeNodePermissionAttr(node, nodeInfo) {
    $(node.span).data("pag", nodeInfo.AddGroupAllowed);
    $(node.span).data("pmg", nodeInfo.ModifyGroupAllowed);
    $(node.span).data("pvg", nodeInfo.ViewGroupAllowed);
    $(node.span).data("pmd", nodeInfo.MoveDevsGroupAllowed);
    $(node.span).data("prd", nodeInfo.RemoveDevsGroupAllowed);
    $(node.span).data("pmo", nodeInfo.ManageOpsDevsGroupAllowed);
    $(node.span).data("pmp", nodeInfo.ModifyPolicyAllowed);
    $(node.span).data("pdp", nodeInfo.DefinePolicyAllowed);
    $(node.span).data("pdip", nodeInfo.DisablePolicyAllowed);
    $(node.span).data("pmanp", nodeInfo.ManagePMAllowed);
    $(node.span).data("pmanav", nodeInfo.ManageAVAllowed);
    $(node.span).data("pmanec", nodeInfo.ManageECAllowed);
}

var $idown;  // Keep it outside of the function, so it's initialized once.
function downloadURL(url) {
    if ($idown) {
        $idown.attr('src', url);
    } else {
        $idown = $('<iframe>', { id: 'idown', src: url }).hide().appendTo('body');
    }
}

function InitialiseCalbackObject(moduleCallbacks, updatedCallbacks) {
    var callbackToUpdate = null;

    if ((updatedCallbacks == null) || (updatedCallbacks.id == null))
        callbackToUpdate = moduleCallbacks[0];
    else
        for (var i = 0; i < moduleCallbacks.length; i++) {
            if (moduleCallbacks[i].id == updatedCallbacks.id)
                callbackToUpdate = moduleCallbacks[i];
        }


    if (callbackToUpdate == null) {
        callbackToUpdate = new Object();

        for (var key in moduleCallbacks[0]) {
            if (updatedCallbacks[key] == null)
                callbackToUpdate[key] = function () { return true; };
            else
                callbackToUpdate[key] = updatedCallbacks[key];
        }

        moduleCallbacks.push(callbackToUpdate);
    }
    else {
        $.extend(callbackToUpdate, updatedCallbacks);

        if (updatedCallbacks != null)
            for (var key in callbackToUpdate)
                if (updatedCallbacks[key] == null)
                    callbackToUpdate[key] = function () { return true; };
    }

    return moduleCallbacks;
}

function generateUUID() {
    var d = new Date().getTime();
    var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = (d + Math.random() * 16) % 16 | 0;
        d = Math.floor(d / 16);
        return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
    return uuid;
};

function GetCalbackObject(moduleCallbacks, objCalling) {
    var callbackId = "";

    if (objCalling != null) {
        var callbackIdLinked = $(objCalling).closest("[callbacks-id]");
        if ((callbackIdLinked.length > 0))
            callbackId = $(callbackIdLinked[0]).attr("callbacks-id");
    }


    if ((moduleCallbacks.length > 1) && (objCalling != null)) {
        for (var i = 0; i < moduleCallbacks.length; i++) {
            if (moduleCallbacks[i].id == callbackId)
                return moduleCallbacks[i];
        }
    }

    return moduleCallbacks[0];
}

function AutocompleteHandler(term, response, container) {
    $.ajax({
        url: ActionURLS.Devices.SearchDevicesForAutoComplete,
        data: {
            startsWith: term
        },
        success: function (result) {
            if (result.Success == "OK") {

                var assignedDevices = [];
                var filteredData = [];
                $(container).find('.content-filter-tag[data-options="devicename"]').find(".filter-tag-value").each(function () {
                    assignedDevices.push($(this).text());
                });

                if (assignedDevices.length > 0) {
                    for (var i = 0; i < result.Devices.length; i++) {
                        if ($.inArray(result.Devices[i].value, assignedDevices) == -1)
                            filteredData.push(result.Devices[i]);
                    }
                }
                else
                    filteredData = result.Devices;

                response(filteredData);
            }
        }
    });
}

/*
    Returns true if result is a success, Else False.
    result - The result
    calledFuncName - The current javascript method
    uiMsgSrc - The message source to be passed on to the show ui message (content, card etc.) If this is null, showUiMsg will not be called
    uiMsgType - The type of message to be passed on to the show ui message (critical, info etc)
    errorMessage - The error message to display in the message ribbon
    MsgTimeToLiveMs - TTL
    callBackFunc - Additional code to be executed when result is not success
*/
function HandleResult(result, calledFuncName, uiMsgSrc, uiMsgType, errorMessage, MsgTimeToLiveMs, callBackFunc) {
    if (result.Success == "OK")
        return true;
    else {
        console && console.log(calledFuncName + " failed: " + errorMessage);

        if (uiMsgSrc != null)
            showUiMsg(uiMsgSrc, uiMsgType, errorMessage, "", "", "", MsgTimeToLiveMs);

        if (callBackFunc != null)
            callBackFunc();

        return false;
    }
}

function HandleAjaxError(xhr, ajaxOptions, thrownError, uiMsgSrc, callingFunctionName, errorMsg) {
    showUiMsg(uiMsgSrc, "critical", errorMsg, "", "");

    console && console.log(callingFunctionName + " failed: " + thrownError);
}

function OpenWindow(url) {
    var w = window.open(url);
    if (!w) {
        //an alert in this example
        alert("Pop-up blocker is enabled. Please visit: " + url);
    }
}
